@@ -121,14 +121,12 @@ extern const int atf_fs_access_r;
 extern const int atf_fs_access_w;
 extern const int atf_fs_access_x;
 
-atf_error_t atf_fs_cleanup(const atf_fs_path_t *);
 atf_error_t atf_fs_eaccess(const atf_fs_path_t *, int);
 atf_error_t atf_fs_exists(const atf_fs_path_t *, bool *);
 atf_error_t atf_fs_getcwd(atf_fs_path_t *);
 atf_error_t atf_fs_mkdtemp(atf_fs_path_t *);
 atf_error_t atf_fs_mkstemp(atf_fs_path_t *, int *);
 atf_error_t atf_fs_rmdir(const atf_fs_path_t *);
-atf_error_t atf_fs_set_immutable(const atf_fs_path_t *, bool, bool *);
 atf_error_t atf_fs_unlink(const atf_fs_path_t *);
 
 #endif /* !defined(ATF_C_FS_H) */
